package com.seeshiontech.vecloudsdk;


import java.io.*;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class Request {

    protected Config config;

    private final String[] remoteHosts = {
            "https://vecloud.atvideo.cc",
            "https://vecloud0.atvideo.cc",
            "https://vecloud1.atvideo.cc",
    };

    public Request(Config config) {
        this.config = config;
    }

    protected String submitTask(String data, String path) {
        String token = genToken(data);
        String authorization = this.config.getProductId() + ":" + this.config.getAccessKey() + ":" + token;
        String response = "";
        for (String url : this.remoteHosts) {
            OutputStreamWriter out = null;
            BufferedReader reader = null;
            HttpURLConnection conn = null;
            try {
                URL httpUrl = null;
                httpUrl = new URL(url + path);

                conn = (HttpURLConnection) httpUrl.openConnection();
                conn.setRequestMethod("POST");
                conn.setRequestProperty("Content-Type", "application/json; charset=utf-8");
                conn.setRequestProperty("Authorization", authorization);
                conn.setUseCaches(false);
                conn.setInstanceFollowRedirects(true);
                conn.setDoOutput(true);
                conn.setDoInput(true);
                conn.connect();
                out = new OutputStreamWriter(conn.getOutputStream(), Charset.forName("UTF-8"));
                out.write(data);
                out.flush();

                reader = new BufferedReader(new InputStreamReader(conn.getInputStream(), Charset.forName("UTF-8")));
                String lines;
                while ((lines = reader.readLine()) != null) {
                    lines = new String(lines.getBytes(), "UTF-8");
                    response += lines;
                }
                System.out.println(response);
                return response;
            } catch (Exception e) {
                continue;
            } finally {
                try {
                    if (reader != null) {
                        reader.close();
                    }
                    if (conn != null) {
                        conn.disconnect();
                    }
                    if (out != null) {
                        out.close();
                        break;
                    }
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }

        return response;
    }

    protected String genToken(String rawData) {
        String hexSecret = md5(this.config.getAccessSecret());
        String encryptString = md5(rawData + "&&" + this.config.getProductId() + "&&" + this.config.getAccessKey() + "&&" + hexSecret);

        return encryptString;
    }

    protected String md5(String plainText) {
        byte[] secretBytes = null;
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            try {
                byte[] bytesText = plainText.getBytes("UTF-8");
                md.update(bytesText);
                secretBytes = md.digest();
            } catch (UnsupportedEncodingException encodingException) {
                throw new RuntimeException("md5 failed");
            }
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("md5 failed");
        }

        String md5code = new BigInteger(1, secretBytes).toString(16);
        for (int i = 0; i < 32 - md5code.length(); i++) {
            md5code = "0" + md5code;
        }
        return md5code;
    }

    public boolean checkParamValid(HashMap<String, Object> taskData, HashMap<String, String> rule) throws Exception {
        for (Map.Entry<String, String> entry : rule.entrySet()) {
            String checkingKey = entry.getKey();
            String checkingValue = entry.getValue();
            String[] checkingRules = checkingValue.split("\\|");
            //taskData 中的value
            Object value = taskData.containsKey(checkingKey) ? taskData.get(checkingKey) : null;

            if (checkingRules[0].equals("required")) {
                if (value == null) {
                    throw new Exception("Missing parameter：" + checkingKey);
                }
                if (checkingRules.length == 3) {
                    Pattern p = Pattern.compile(checkingRules[2]);
                    Matcher m = p.matcher(value.toString());

                    if (m.matches()) {
                        throw new Exception("Param rule failed :" + checkingKey);
                    }
                }
            }

            if (value != null) {
                if (checkingRules[1].equals("string") && !(value instanceof String)) {
                    throw new Exception(checkingKey + " 's type must be string");
                }
                if (checkingRules[1].equals("int") && !(value instanceof Integer)) {
                    throw new Exception(checkingKey + " 's type must be int");
                }
                if (checkingRules[1].equals("array") && !(value instanceof ArrayList)) {
                    throw new Exception(checkingKey + " 's type must be ArrayList");
                }
                if (checkingRules[1].equals("float") && !(isDoubleOrFloat(value.toString()))) {
                    throw new Exception(checkingKey + " 's type must be float");
                }
            }
        }
        return true;
    }

    public static boolean isDoubleOrFloat(String str) {
        Pattern pattern = Pattern.compile("^[-\\+]?[.\\d]*$");
        return pattern.matcher(str).matches();
    }

}
